
pro crossCorrelate, X, y, tvals, ccf, $
        varx=varx, vary=vary, ftx=ftx, fty=fty
;+
;PURPOSE: calculate CCF of time series X and Y using FFT. Also provides the
;FFTs of X and Y.  Stolen from IDL documentation of A_CORRELATE. 
;
;Note this subtracts the mean beforehand, as does C_CORRELATE. Note that
;this result, using FFT, differs somewhat from that using C_CORRELATE.
;
;CALLING SEQUENCE:
;CROSSCORRELATE, X, y, tvals, ccf, $
;        varx=varx, vary=vary, ftx=ftx, fty=fty
;
;INPUTS:
;X, Y, the time series, each array has N points.
;
;OUTPUTS:
;CCF  We return an array of real numbers, because the
;       imaginary parts are all zero. the CCF has N points
;TVALS, the delays, which run from -N/2 to N/2. 
;FTX, FTY, the FFTs of X and Y. Complex arrays with N points each
;VARX, VARY, the variances of X and Y. Multiply ACF by VARIANCE to get the
;       covariance. The intensity scaling of CCF is the same as it would
;       be for ACF, i.e. perfect corellation gives CCF=1.0
;-

nrp=n_elements( x)

varx=variance(x)
vary=variance(y)
vxvy= sqrt( varx*vary)

ftx = FFT(X - mean(x),-1)   
fty = FFT(y - mean(y),-1)   

ccf= real_part(FFT(Fty * CONJ(Ftx), 1))/vxvy
tvals=shift( lindgen( nrp)-nrp/2l, nrp/2)

END
