pro fft_freqs_times, fsmpl, nrsmpl, freqs, times, $
        fmax=fmax, tmax=tmax, tsmpl=tsmpl

;+
;Calling sequence:
;       fft_freqs_times, fsmpl, nrsmpl, freqs, times, $
;        fmax=fmax, tmax=tmax, tsmpl=tsmpl
;
;For a time series sampled with sampling freq FSMPL and NRSMPL points,
;for which you will do an FFT, return the frequencies in ensuing spectrum
;and the times in the time series.
;
;INPUTS:
;FSMPL, the sampling frequency
;NRSMPL, the number of points in the time series
;
;KEYWORDS:
;FMAX. Set it for the freq array to cover -[(FSMPL/2)+DF] to +[FSMPL/2];
;default range is -[FSMPL/2] to +[(FSMPL/2)-DF]
;TMAX. Set it for the times array to cover -[(TSMPL/2)+DF] to +[TSMPL/2];
;default range is -[TSMPL/2] to +[(TSMPL/2)-DF]
;
;OUTPUTS
;FREQS, tne frequencies in the spectrum. 
;-- There are NRSMPL points in this spectrum. 
;-- the spectral points are separated by DF=(FSMPL/NRSMPL)
;-- the lowest frequency is FMIN=-(FSMPL/2)
;-- the highest frequency is FMAX=+(FSMPL/2)-DF
;-- There are NRSMPL negative freqs and NRSMPL-1 positive freqs; the
;   other point has freq=0.
;-- the freqs array is ordered like the FFT output: 
;---------- the array begins at 0
;---------- the goes with positive frequencies up to FMAX (that's (NRSMPL/2) spectral points)
;---------- The next spectral point (number (NRSMPL/2)+1) has freq FMIN
;             it can also be ragarded as having freq FMAX. Set the
;             keyword FMAX if you want it's freq to be labelled FMAX.
;---------- the array continues with neg freqs, a total of NRSMPL/2-1
;               negative frequencies
;---------- The last elelment has freq=-DF.

;times, tne times in the time series
;-- There are NRSMPL points in this time series. 
;-- the time series times are separated by DT=TSMPL, where TSMPL=1/FSMPL
;-- the lowest time is TMIN=-(TSMPL*NRMPL/2)
;-- the highest time is TMAX=+(TSMPL*NRSMPL/2)-DT
;-- There are NRSMPL negative times and NRSMPL-1 positive times; the
;   other point has time=0.
;-- the times array is ordered like the FFT input: 
;---------- the array begins at 0
;---------- it goes with positive times up to TMAX (that's
;                (NRSMPL/2) time series points)
;---------- The next time series point (number (NRSMPL/2)+1) has time TMIN.
;---------- the array continues with neg times, a total of NRSMPL/2-1
;---------- negative times
;---------- The last elelment has time=-DT.
;-

tsmpl= 1.d0/fsmpl

freqs= fsmpl* (dindgen(nrsmpl) - nrsmpl/2)/nrsmpl
times= tsmpl* (dindgen(nrsmpl) - nrsmpl/2)

freqs= shift(freqs, nrsmpl/2)
times= shift(times, nrsmpl/2)

if keyword_set( fmax) then begin
   freqs[nrsmpl/2]=-freqs[nrsmpl/2]
   times[nrsmpl/2]=-times[nrsmpl/2]
endif

return
end
